/**
 * Weather Source Climatology API
 * Daily and hourly climatological weather data for latitude/longitude points, ZIP/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source Climatology API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 *//* tslint:disable:no-unused-variable member-ordering */

import { Inject, Injectable, Optional }                      from '@angular/core';
import { HttpClient, HttpHeaders, HttpParams,
         HttpResponse, HttpEvent }                           from '@angular/common/http';
import { CustomHttpUrlEncodingCodec }                        from '../encoder';

import { Observable }                                        from 'rxjs';

import { ClimatologyPostcodeDayObj } from '../model/climatologyPostcodeDayObj';
import { ClimatologyPostcodeHourObj } from '../model/climatologyPostcodeHourObj';
import { ErrorObj } from '../model/errorObj';

import { BASE_PATH, COLLECTION_FORMATS }                     from '../variables';
import { Configuration }                                     from '../configuration';


@Injectable()
export class PostcodesService {

    protected basePath = 'https://climatology.weathersourceapis.com/v2';
    public defaultHeaders = new HttpHeaders();
    public configuration = new Configuration();

    constructor(protected httpClient: HttpClient, @Optional()@Inject(BASE_PATH) basePath: string, @Optional() configuration: Configuration) {
        if (basePath) {
            this.basePath = basePath;
        }
        if (configuration) {
            this.configuration = configuration;
            this.basePath = basePath || configuration.basePath || this.basePath;
        }
    }

    /**
     * @param consumes string[] mime-types
     * @return true: consumes contains 'multipart/form-data', false: otherwise
     */
    private canConsumeForm(consumes: string[]): boolean {
        const form = 'multipart/form-data';
        for (const consume of consumes) {
            if (form === consume) {
                return true;
            }
        }
        return false;
    }


    /**
     * A day of weather climatology for a postcode.
     * Returns a single day of climatological weather data for a postcode.
     * @param postcode A postcode string.
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
     * @param date An date formatted as a RFC3339 date value.
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, temp, wetBulb)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
     * @param unitScale The unit scale for returned values.&lt;br&gt;
     * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
     * @param reportProgress flag to report request and response progress.
     */
    public postcodesPostcodecountryCodeDaysDateGet(postcode: string, countryCode: string, date: string, fields: Array<string>, unitScale?: string, observe?: 'body', reportProgress?: boolean): Observable<ClimatologyPostcodeDayObj>;
    public postcodesPostcodecountryCodeDaysDateGet(postcode: string, countryCode: string, date: string, fields: Array<string>, unitScale?: string, observe?: 'response', reportProgress?: boolean): Observable<HttpResponse<ClimatologyPostcodeDayObj>>;
    public postcodesPostcodecountryCodeDaysDateGet(postcode: string, countryCode: string, date: string, fields: Array<string>, unitScale?: string, observe?: 'events', reportProgress?: boolean): Observable<HttpEvent<ClimatologyPostcodeDayObj>>;
    public postcodesPostcodecountryCodeDaysDateGet(postcode: string, countryCode: string, date: string, fields: Array<string>, unitScale?: string, observe: any = 'body', reportProgress: boolean = false ): Observable<any> {

        if (postcode === null || postcode === undefined) {
            throw new Error('Required parameter postcode was null or undefined when calling postcodesPostcodecountryCodeDaysDateGet.');
        }

        if (countryCode === null || countryCode === undefined) {
            throw new Error('Required parameter countryCode was null or undefined when calling postcodesPostcodecountryCodeDaysDateGet.');
        }

        if (date === null || date === undefined) {
            throw new Error('Required parameter date was null or undefined when calling postcodesPostcodecountryCodeDaysDateGet.');
        }

        if (fields === null || fields === undefined) {
            throw new Error('Required parameter fields was null or undefined when calling postcodesPostcodecountryCodeDaysDateGet.');
        }


        let queryParameters = new HttpParams({encoder: new CustomHttpUrlEncodingCodec()});
        if (fields) {
            queryParameters = queryParameters.set('fields', fields.join(COLLECTION_FORMATS['csv']));
        }
        if (unitScale !== undefined && unitScale !== null) {
            queryParameters = queryParameters.set('unitScale', <any>unitScale);
        }

        let headers = this.defaultHeaders;

        // authentication (ApiKeyAuth) required
        if (this.configuration.apiKeys && this.configuration.apiKeys["X-API-KEY"]) {
            headers = headers.set('X-API-KEY', this.configuration.apiKeys["X-API-KEY"]);
        }

        // to determine the Accept header
        let httpHeaderAccepts: string[] = [
            'application/json'
        ];
        const httpHeaderAcceptSelected: string | undefined = this.configuration.selectHeaderAccept(httpHeaderAccepts);
        if (httpHeaderAcceptSelected != undefined) {
            headers = headers.set('Accept', httpHeaderAcceptSelected);
        }

        // to determine the Content-Type header
        const consumes: string[] = [
        ];

        return this.httpClient.request<ClimatologyPostcodeDayObj>('get',`${this.basePath}/postcodes/${encodeURIComponent(String(postcode))},${encodeURIComponent(String(countryCode))}/days/${encodeURIComponent(String(date))}`,
            {
                params: queryParameters,
                withCredentials: this.configuration.withCredentials,
                headers: headers,
                observe: observe,
                reportProgress: reportProgress
            }
        );
    }

    /**
     * A contiguous range of daily weather climatology for a postcode.
     * Returns a contiguous range of daily climatological weather data for a postcode.
     * @param postcode A postcode string.
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd.
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart.
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, temp, wetBulb)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
     * @param unitScale The unit scale for returned values.&lt;br&gt;
     * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
     * @param reportProgress flag to report request and response progress.
     */
    public postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode: string, countryCode: string, dateStart: string, dateEnd: string, fields: Array<string>, unitScale?: string, observe?: 'body', reportProgress?: boolean): Observable<ClimatologyPostcodeDayObj>;
    public postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode: string, countryCode: string, dateStart: string, dateEnd: string, fields: Array<string>, unitScale?: string, observe?: 'response', reportProgress?: boolean): Observable<HttpResponse<ClimatologyPostcodeDayObj>>;
    public postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode: string, countryCode: string, dateStart: string, dateEnd: string, fields: Array<string>, unitScale?: string, observe?: 'events', reportProgress?: boolean): Observable<HttpEvent<ClimatologyPostcodeDayObj>>;
    public postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode: string, countryCode: string, dateStart: string, dateEnd: string, fields: Array<string>, unitScale?: string, observe: any = 'body', reportProgress: boolean = false ): Observable<any> {

        if (postcode === null || postcode === undefined) {
            throw new Error('Required parameter postcode was null or undefined when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet.');
        }

        if (countryCode === null || countryCode === undefined) {
            throw new Error('Required parameter countryCode was null or undefined when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet.');
        }

        if (dateStart === null || dateStart === undefined) {
            throw new Error('Required parameter dateStart was null or undefined when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet.');
        }

        if (dateEnd === null || dateEnd === undefined) {
            throw new Error('Required parameter dateEnd was null or undefined when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet.');
        }

        if (fields === null || fields === undefined) {
            throw new Error('Required parameter fields was null or undefined when calling postcodesPostcodecountryCodeDaysDateStartdateEndGet.');
        }


        let queryParameters = new HttpParams({encoder: new CustomHttpUrlEncodingCodec()});
        if (fields) {
            queryParameters = queryParameters.set('fields', fields.join(COLLECTION_FORMATS['csv']));
        }
        if (unitScale !== undefined && unitScale !== null) {
            queryParameters = queryParameters.set('unitScale', <any>unitScale);
        }

        let headers = this.defaultHeaders;

        // authentication (ApiKeyAuth) required
        if (this.configuration.apiKeys && this.configuration.apiKeys["X-API-KEY"]) {
            headers = headers.set('X-API-KEY', this.configuration.apiKeys["X-API-KEY"]);
        }

        // to determine the Accept header
        let httpHeaderAccepts: string[] = [
            'application/json'
        ];
        const httpHeaderAcceptSelected: string | undefined = this.configuration.selectHeaderAccept(httpHeaderAccepts);
        if (httpHeaderAcceptSelected != undefined) {
            headers = headers.set('Accept', httpHeaderAcceptSelected);
        }

        // to determine the Content-Type header
        const consumes: string[] = [
        ];

        return this.httpClient.request<ClimatologyPostcodeDayObj>('get',`${this.basePath}/postcodes/${encodeURIComponent(String(postcode))},${encodeURIComponent(String(countryCode))}/days/${encodeURIComponent(String(dateStart))},${encodeURIComponent(String(dateEnd))}`,
            {
                params: queryParameters,
                withCredentials: this.configuration.withCredentials,
                headers: headers,
                observe: observe,
                reportProgress: reportProgress
            }
        );
    }

    /**
     * An hour of weather climatology for a postcode.
     * Returns a single hour of climatological weather data for a postcode.
     * @param postcode A postcode string.
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(derivatives of precip, snowfall)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, temp, wetBulb)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windVecSpd, windVecSpd80m, windVecSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
     * @param unitScale The unit scale for returned values.&lt;br&gt;
     * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
     * @param reportProgress flag to report request and response progress.
     */
    public postcodesPostcodecountryCodeHoursTimestampGet(postcode: string, countryCode: string, timestamp: Date, fields: Array<string>, unitScale?: string, observe?: 'body', reportProgress?: boolean): Observable<ClimatologyPostcodeHourObj>;
    public postcodesPostcodecountryCodeHoursTimestampGet(postcode: string, countryCode: string, timestamp: Date, fields: Array<string>, unitScale?: string, observe?: 'response', reportProgress?: boolean): Observable<HttpResponse<ClimatologyPostcodeHourObj>>;
    public postcodesPostcodecountryCodeHoursTimestampGet(postcode: string, countryCode: string, timestamp: Date, fields: Array<string>, unitScale?: string, observe?: 'events', reportProgress?: boolean): Observable<HttpEvent<ClimatologyPostcodeHourObj>>;
    public postcodesPostcodecountryCodeHoursTimestampGet(postcode: string, countryCode: string, timestamp: Date, fields: Array<string>, unitScale?: string, observe: any = 'body', reportProgress: boolean = false ): Observable<any> {

        if (postcode === null || postcode === undefined) {
            throw new Error('Required parameter postcode was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampGet.');
        }

        if (countryCode === null || countryCode === undefined) {
            throw new Error('Required parameter countryCode was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampGet.');
        }

        if (timestamp === null || timestamp === undefined) {
            throw new Error('Required parameter timestamp was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampGet.');
        }

        if (fields === null || fields === undefined) {
            throw new Error('Required parameter fields was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampGet.');
        }


        let queryParameters = new HttpParams({encoder: new CustomHttpUrlEncodingCodec()});
        if (fields) {
            queryParameters = queryParameters.set('fields', fields.join(COLLECTION_FORMATS['csv']));
        }
        if (unitScale !== undefined && unitScale !== null) {
            queryParameters = queryParameters.set('unitScale', <any>unitScale);
        }

        let headers = this.defaultHeaders;

        // authentication (ApiKeyAuth) required
        if (this.configuration.apiKeys && this.configuration.apiKeys["X-API-KEY"]) {
            headers = headers.set('X-API-KEY', this.configuration.apiKeys["X-API-KEY"]);
        }

        // to determine the Accept header
        let httpHeaderAccepts: string[] = [
            'application/json'
        ];
        const httpHeaderAcceptSelected: string | undefined = this.configuration.selectHeaderAccept(httpHeaderAccepts);
        if (httpHeaderAcceptSelected != undefined) {
            headers = headers.set('Accept', httpHeaderAcceptSelected);
        }

        // to determine the Content-Type header
        const consumes: string[] = [
        ];

        return this.httpClient.request<ClimatologyPostcodeHourObj>('get',`${this.basePath}/postcodes/${encodeURIComponent(String(postcode))},${encodeURIComponent(String(countryCode))}/hours/${encodeURIComponent(String(timestamp))}`,
            {
                params: queryParameters,
                withCredentials: this.configuration.withCredentials,
                headers: headers,
                observe: observe,
                reportProgress: reportProgress
            }
        );
    }

    /**
     * A contiguous range of hourly weather climatology for a postcode.
     * Returns a contiguous range of hourly climatological weather data for a postcode.
     * @param postcode A postcode string.
     * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(derivatives of precip, snowfall)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, temp, wetBulb)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windVecSpd, windVecSpd80m, windVecSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
     * @param unitScale The unit scale for returned values.&lt;br&gt;
     * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
     * @param reportProgress flag to report request and response progress.
     */
    public postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode: string, countryCode: string, timestampStart: Date, timestampEnd: Date, fields: Array<string>, unitScale?: string, observe?: 'body', reportProgress?: boolean): Observable<ClimatologyPostcodeHourObj>;
    public postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode: string, countryCode: string, timestampStart: Date, timestampEnd: Date, fields: Array<string>, unitScale?: string, observe?: 'response', reportProgress?: boolean): Observable<HttpResponse<ClimatologyPostcodeHourObj>>;
    public postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode: string, countryCode: string, timestampStart: Date, timestampEnd: Date, fields: Array<string>, unitScale?: string, observe?: 'events', reportProgress?: boolean): Observable<HttpEvent<ClimatologyPostcodeHourObj>>;
    public postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode: string, countryCode: string, timestampStart: Date, timestampEnd: Date, fields: Array<string>, unitScale?: string, observe: any = 'body', reportProgress: boolean = false ): Observable<any> {

        if (postcode === null || postcode === undefined) {
            throw new Error('Required parameter postcode was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet.');
        }

        if (countryCode === null || countryCode === undefined) {
            throw new Error('Required parameter countryCode was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet.');
        }

        if (timestampStart === null || timestampStart === undefined) {
            throw new Error('Required parameter timestampStart was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet.');
        }

        if (timestampEnd === null || timestampEnd === undefined) {
            throw new Error('Required parameter timestampEnd was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet.');
        }

        if (fields === null || fields === undefined) {
            throw new Error('Required parameter fields was null or undefined when calling postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet.');
        }


        let queryParameters = new HttpParams({encoder: new CustomHttpUrlEncodingCodec()});
        if (fields) {
            queryParameters = queryParameters.set('fields', fields.join(COLLECTION_FORMATS['csv']));
        }
        if (unitScale !== undefined && unitScale !== null) {
            queryParameters = queryParameters.set('unitScale', <any>unitScale);
        }

        let headers = this.defaultHeaders;

        // authentication (ApiKeyAuth) required
        if (this.configuration.apiKeys && this.configuration.apiKeys["X-API-KEY"]) {
            headers = headers.set('X-API-KEY', this.configuration.apiKeys["X-API-KEY"]);
        }

        // to determine the Accept header
        let httpHeaderAccepts: string[] = [
            'application/json'
        ];
        const httpHeaderAcceptSelected: string | undefined = this.configuration.selectHeaderAccept(httpHeaderAccepts);
        if (httpHeaderAcceptSelected != undefined) {
            headers = headers.set('Accept', httpHeaderAcceptSelected);
        }

        // to determine the Content-Type header
        const consumes: string[] = [
        ];

        return this.httpClient.request<ClimatologyPostcodeHourObj>('get',`${this.basePath}/postcodes/${encodeURIComponent(String(postcode))},${encodeURIComponent(String(countryCode))}/hours/${encodeURIComponent(String(timestampStart))},${encodeURIComponent(String(timestampEnd))}`,
            {
                params: queryParameters,
                withCredentials: this.configuration.withCredentials,
                headers: headers,
                observe: observe,
                reportProgress: reportProgress
            }
        );
    }

}
